@@ -45,8 +45,8 @@ static uint64_t
 read_counter(struct etna_context *ctx, unsigned type)
 {
    switch (type) {
-   case PIPE_QUERY_PRIMITIVES_EMITTED:
-      return ctx->stats.prims_emitted;
+   case PIPE_QUERY_PRIMITIVES_GENERATED:
+      return ctx->stats.prims_generated;
    case ETNA_QUERY_DRAW_CALLS:
       return ctx->stats.draw_calls;
    case ETNA_QUERY_RS_OPERATIONS:
@@ -97,7 +97,7 @@ etna_sw_create_query(struct etna_context *ctx, unsigned query_type)
    struct etna_query *q;
 
    switch (query_type) {
-   case PIPE_QUERY_PRIMITIVES_EMITTED:
+   case PIPE_QUERY_PRIMITIVES_GENERATED:
    case ETNA_QUERY_DRAW_CALLS:
    case ETNA_QUERY_RS_OPERATIONS:
       break;
@@ -117,7 +117,7 @@ etna_sw_create_query(struct etna_context *ctx, unsigned query_type)
 }
 
 static const struct pipe_driver_query_info list[] = {
-   {"prims-emitted", PIPE_QUERY_PRIMITIVES_EMITTED, { 0 }},
+   {"prims-generated", PIPE_QUERY_PRIMITIVES_GENERATED, { 0 }},
    {"draw-calls", ETNA_QUERY_DRAW_CALLS, { 0 }},
    {"rs-operations", ETNA_QUERY_RS_OPERATIONS, { 0 }},
 };
