@@ -203,19 +203,14 @@ bool InlineConstValue::is_equal_to(const Value& other) const
 UniformValue::UniformValue(uint32_t sel, uint32_t chan, uint32_t kcache_bank):
    Value(Value::kconst, chan)
 {
-   if (sel < 512) {
-      m_index = sel & 0x1f;
-      m_kcache_bank = ((sel >> 5) & 1) |  ((sel >> 7) & 2);
-   } else {
-      m_index = sel;
-      m_kcache_bank = kcache_bank;
-   }
+   m_index = sel;
+   m_kcache_bank = kcache_bank;
 }
 
 UniformValue::UniformValue(uint32_t sel, uint32_t chan, PValue addr):
    Value(Value::kconst, chan),
    m_index(sel),
-   m_kcache_bank(0),
+   m_kcache_bank(1),
    m_addr(addr)
 {
 
@@ -243,8 +238,10 @@ void UniformValue::do_print(std::ostream& os) const
 {
    if (m_index < 512)
       os << "KC" << m_kcache_bank << "[" << m_index;
+   else if (m_addr)
+      os << "KC[" << *m_addr << "][" << m_index;
    else
-      os << "KCX[" << m_index;
+      os << "KCx[" << m_index;
    os << "]." << component_names[chan()];
 }
 
