@@ -55,7 +55,7 @@ load_color_map_texture(struct gl_context *ctx, struct pipe_resource *pt)
    uint i, j;
 
    dest = (uint *) pipe_transfer_map(pipe,
-                                     pt, 0, 0, PIPE_TRANSFER_WRITE,
+                                     pt, 0, 0, PIPE_MAP_WRITE,
                                      0, 0, texSize, texSize, &transfer);
 
    /* Pack four 1D maps into a 2D texture:
@@ -84,8 +84,8 @@ load_color_map_texture(struct gl_context *ctx, struct pipe_resource *pt)
 /**
  * Upload the pixel transfer color map texture.
  */
-static void
-update_pixel_transfer(struct st_context *st)
+void
+st_update_pixel_transfer(struct st_context *st)
 {
    struct gl_context *ctx = st->ctx;
 
@@ -100,8 +100,3 @@ update_pixel_transfer(struct st_context *st)
       load_color_map_texture(ctx, st->pixel_xfer.pixelmap_texture);
    }
 }
-
-
-const struct st_tracked_state st_update_pixel_transfer = {
-   update_pixel_transfer				/* update */
-};
