@@ -150,7 +150,7 @@ make_drawpix_z_stencil_program_nir(struct st_context *st,
 {
    struct nir_builder b;
    const nir_shader_compiler_options *options =
-      st->ctx->Const.ShaderCompilerOptions[MESA_SHADER_FRAGMENT].NirOptions;
+      st_get_nir_compiler_options(st, MESA_SHADER_FRAGMENT);
 
    nir_builder_init_simple_shader(&b, NULL, MESA_SHADER_FRAGMENT, options);
 
@@ -198,65 +198,63 @@ make_drawpix_z_stencil_program_nir(struct st_context *st,
    return st_nir_finish_builtin_shader(st, b.shader, name);
 }
 
-
 static void *
-make_drawpix_z_stencil_program_tgsi(struct st_context *st,
-                                    bool write_depth,
-                                    bool write_stencil)
+make_drawpix_zs_to_color_program_nir(struct st_context *st,
+                                   bool rgba)
 {
-   struct ureg_program *ureg;
-   struct ureg_src depth_sampler, stencil_sampler;
-   struct ureg_src texcoord, color;
-   struct ureg_dst out_color, out_depth, out_stencil;
+   struct nir_builder b;
+   const nir_shader_compiler_options *options =
+      st_get_nir_compiler_options(st, MESA_SHADER_FRAGMENT);
 
-   ureg = ureg_create(PIPE_SHADER_FRAGMENT);
-   if (ureg == NULL)
-      return NULL;
+   nir_builder_init_simple_shader(&b, NULL, MESA_SHADER_FRAGMENT, options);
 
-   ureg_property(ureg, TGSI_PROPERTY_FS_COLOR0_WRITES_ALL_CBUFS, TRUE);
+   nir_variable *texcoord =
+      nir_variable_create(b.shader, nir_var_shader_in, glsl_vec_type(2),
+                          "texcoord");
+   texcoord->data.location = VARYING_SLOT_TEX0;
 
-   if (write_depth) {
-      color = ureg_DECL_fs_input(ureg, TGSI_SEMANTIC_COLOR, 0,
-                                 TGSI_INTERPOLATE_COLOR);
-      out_color = ureg_DECL_output(ureg, TGSI_SEMANTIC_COLOR, 0);
+   /* Sample depth and stencil */
+   nir_ssa_def *depth = sample_via_nir(&b, texcoord, "depth", 0,
+                                       GLSL_TYPE_FLOAT, nir_type_float);
+   nir_ssa_def *stencil = sample_via_nir(&b, texcoord, "stencil", 1,
+                                         GLSL_TYPE_UINT, nir_type_uint);
 
-      depth_sampler = ureg_DECL_sampler(ureg, 0);
-      ureg_DECL_sampler_view(ureg, 0, TGSI_TEXTURE_2D,
-                             TGSI_RETURN_TYPE_FLOAT,
-                             TGSI_RETURN_TYPE_FLOAT,
-                             TGSI_RETURN_TYPE_FLOAT,
-                             TGSI_RETURN_TYPE_FLOAT);
-      out_depth = ureg_DECL_output(ureg, TGSI_SEMANTIC_POSITION, 0);
+   /* Create the variable to store the output color */
+   nir_variable *color_out =
+      nir_variable_create(b.shader, nir_var_shader_out, glsl_vec_type(4),
+                          "make_drawpix_zs_to_color_program_nirgl_FragColor");
+   color_out->data.location = FRAG_RESULT_COLOR;
+
+   nir_ssa_def *shifted_depth = nir_fmul(&b,nir_f2f64(&b, depth), nir_imm_double(&b,0xffffff));
+   nir_ssa_def *int_depth = nir_f2u32(&b,shifted_depth);
+
+   nir_ssa_def *ds[4];
+   ds[0] = nir_ubitfield_extract(&b, stencil, nir_imm_int(&b, 0), nir_imm_int(&b,8));
+   ds[1] = nir_ubitfield_extract(&b, int_depth, nir_imm_int(&b, 0), nir_imm_int(&b,8));
+   ds[2] = nir_ubitfield_extract(&b, int_depth, nir_imm_int(&b, 8), nir_imm_int(&b,8));
+   ds[3] = nir_ubitfield_extract(&b, int_depth, nir_imm_int(&b, 16), nir_imm_int(&b,8));
+
+   nir_ssa_def *ds_comp[4];
+   ds_comp[0] = nir_fsat(&b, nir_fmul_imm(&b, nir_u2f32(&b, ds[3]), 1.0/255.0));
+   ds_comp[1] = nir_fsat(&b, nir_fmul_imm(&b, nir_u2f32(&b, ds[2]), 1.0/255.0));
+   ds_comp[2] = nir_fsat(&b, nir_fmul_imm(&b, nir_u2f32(&b, ds[1]), 1.0/255.0));
+   ds_comp[3] = nir_fsat(&b, nir_fmul_imm(&b, nir_u2f32(&b, ds[0]), 1.0/255.0));
+
+   nir_ssa_def *unpacked_ds = nir_vec4(&b, ds_comp[0], ds_comp[1], ds_comp[2], ds_comp[3]);
+
+   if (rgba) {
+      nir_store_var(&b, color_out, unpacked_ds, 0xf);
+   }
+   else {
+      unsigned zyxw[4] = { 2, 1, 0, 3 };
+      nir_ssa_def *swizzled_ds= nir_swizzle(&b, unpacked_ds, zyxw, 4);
+      nir_store_var(&b, color_out, swizzled_ds, 0xf);
    }
 
-   if (write_stencil) {
-      stencil_sampler = ureg_DECL_sampler(ureg, 1);
-      ureg_DECL_sampler_view(ureg, 1, TGSI_TEXTURE_2D,
-                             TGSI_RETURN_TYPE_UINT,
-                             TGSI_RETURN_TYPE_UINT,
-                             TGSI_RETURN_TYPE_UINT,
-                             TGSI_RETURN_TYPE_UINT);
-      out_stencil = ureg_DECL_output(ureg, TGSI_SEMANTIC_STENCIL, 0);
-   }
+   char name[17];
+   snprintf(name, 17, "copypixels ZStoC");
 
-   texcoord = ureg_DECL_fs_input(ureg,
-                                 st->needs_texcoord_semantic ?
-                                    TGSI_SEMANTIC_TEXCOORD :
-                                    TGSI_SEMANTIC_GENERIC,
-                                 0, TGSI_INTERPOLATE_LINEAR);
-
-   if (write_depth) {
-      ureg_TEX(ureg, ureg_writemask(out_depth, TGSI_WRITEMASK_Z),
-               TGSI_TEXTURE_2D, texcoord, depth_sampler);
-      ureg_MOV(ureg, out_color, color);
-   }
-
-   if (write_stencil)
-      ureg_TEX(ureg, ureg_writemask(out_stencil, TGSI_WRITEMASK_Y),
-               TGSI_TEXTURE_2D, texcoord, stencil_sampler);
-
-   ureg_END(ureg);
-   return ureg_create_shader_and_destroy(ureg, st->pipe);
+   return st_nir_finish_builtin_shader(st, b.shader, name);
 }
 
 
@@ -273,7 +271,6 @@ get_drawpix_z_stencil_program(struct st_context *st,
                               bool write_depth,
                               bool write_stencil)
 {
-   struct pipe_screen *pscreen = st->pipe->screen;
    const GLuint shaderIndex = write_depth * 2 + write_stencil;
    void *cso;
 
@@ -284,20 +281,45 @@ get_drawpix_z_stencil_program(struct st_context *st,
       return st->drawpix.zs_shaders[shaderIndex];
    }
 
-   enum pipe_shader_ir preferred_ir =
-      pscreen->get_shader_param(pscreen, PIPE_SHADER_FRAGMENT,
-                                PIPE_SHADER_CAP_PREFERRED_IR);
-
-   if (preferred_ir == PIPE_SHADER_IR_NIR)
-      cso = make_drawpix_z_stencil_program_nir(st, write_depth, write_stencil);
-   else
-      cso = make_drawpix_z_stencil_program_tgsi(st, write_depth, write_stencil);
+   cso = make_drawpix_z_stencil_program_nir(st, write_depth, write_stencil);
 
    /* save the new shader */
    st->drawpix.zs_shaders[shaderIndex] = cso;
    return cso;
 }
 
+/**
+ * Create fragment program that does a TEX() instruction to get a Z and
+ * stencil value value, then writes to FRAG_RESULT_COLOR.
+ * Used for glCopyPixels(GL_DEPTH_STENCIL_TO_RGBA_NV / GL_DEPTH_STENCIL_TO_BGRA_NV).
+ *
+ * \return CSO of the fragment shader.
+ */
+static void *
+get_drawpix_zs_to_color_program(struct st_context *st,
+                              bool rgba)
+{
+   void *cso;
+   GLuint shaderIndex;
+
+   if (rgba)
+      shaderIndex = 4;
+   else
+      shaderIndex = 5;
+
+   assert(shaderIndex < ARRAY_SIZE(st->drawpix.zs_shaders));
+
+   if (st->drawpix.zs_shaders[shaderIndex]) {
+      /* already have the proper shader */
+      return st->drawpix.zs_shaders[shaderIndex];
+   }
+
+   cso = make_drawpix_zs_to_color_program_nir(st, rgba);
+
+   /* save the new shader */
+   st->drawpix.zs_shaders[shaderIndex] = cso;
+   return cso;
+}
 
 /**
  * Create a simple vertex shader that just passes through the
@@ -306,39 +328,18 @@ get_drawpix_z_stencil_program(struct st_context *st,
 void
 st_make_passthrough_vertex_shader(struct st_context *st)
 {
-   struct pipe_context *pipe = st->pipe;
-   struct pipe_screen *screen = pipe->screen;
-
    if (st->passthrough_vs)
       return;
 
-   enum pipe_shader_ir preferred_ir =
-      screen->get_shader_param(screen, PIPE_SHADER_VERTEX,
-                               PIPE_SHADER_CAP_PREFERRED_IR);
+   unsigned inputs[] =
+      {  VERT_ATTRIB_POS, VERT_ATTRIB_COLOR0, VERT_ATTRIB_GENERIC0 };
+   unsigned outputs[] =
+      { VARYING_SLOT_POS,  VARYING_SLOT_COL0,    VARYING_SLOT_TEX0 };
 
-   if (preferred_ir == PIPE_SHADER_IR_NIR) {
-      unsigned inputs[] =
-         {  VERT_ATTRIB_POS, VERT_ATTRIB_COLOR0, VERT_ATTRIB_GENERIC0 };
-      unsigned outputs[] =
-         { VARYING_SLOT_POS,  VARYING_SLOT_COL0,    VARYING_SLOT_TEX0 };
-
-      st->passthrough_vs =
-         st_nir_make_passthrough_shader(st, "drawpixels VS",
-                                        MESA_SHADER_VERTEX, 3,
-                                        inputs, outputs, NULL, 0);
-   } else {
-      const enum tgsi_semantic semantic_names[] = {
-         TGSI_SEMANTIC_POSITION,
-         TGSI_SEMANTIC_COLOR,
-         st->needs_texcoord_semantic ? TGSI_SEMANTIC_TEXCOORD :
-                                       TGSI_SEMANTIC_GENERIC
-      };
-      const uint semantic_indexes[] = { 0, 0, 0 };
-
-      st->passthrough_vs =
-         util_make_vertex_passthrough_shader(st->pipe, 3, semantic_names,
-                                             semantic_indexes, false);
-   }
+   st->passthrough_vs =
+      st_nir_make_passthrough_shader(st, "drawpixels VS",
+                                     MESA_SHADER_VERTEX, 3,
+                                     inputs, outputs, NULL, 0);
 }
 
 
@@ -674,7 +675,7 @@ make_texture(struct st_context *st,
 
       /* map texture transfer */
       dest = pipe_transfer_map(pipe, pt, 0, 0,
-                               PIPE_TRANSFER_WRITE, 0, 0,
+                               PIPE_MAP_WRITE, 0, 0,
                                width, height, &transfer);
       if (!dest) {
          pipe_resource_reference(&pt, NULL);
@@ -954,7 +955,7 @@ draw_stencil_pixels(struct gl_context *ctx, GLint x, GLint y,
    struct st_context *st = st_context(ctx);
    struct pipe_context *pipe = st->pipe;
    struct st_renderbuffer *strb;
-   enum pipe_transfer_usage usage;
+   enum pipe_map_flags usage;
    struct pipe_transfer *pt;
    const GLboolean zoom = ctx->Pixel.ZoomX != 1.0 || ctx->Pixel.ZoomY != 1.0;
    ubyte *stmap;
@@ -980,10 +981,10 @@ draw_stencil_pixels(struct gl_context *ctx, GLint x, GLint y,
    if (format == GL_STENCIL_INDEX &&
        _mesa_is_format_packed_depth_stencil(strb->Base.Format)) {
       /* writing stencil to a combined depth+stencil buffer */
-      usage = PIPE_TRANSFER_READ_WRITE;
+      usage = PIPE_MAP_READ_WRITE;
    }
    else {
-      usage = PIPE_TRANSFER_WRITE;
+      usage = PIPE_MAP_WRITE;
    }
 
    stmap = pipe_transfer_map(pipe, strb->texture,
@@ -1041,7 +1042,7 @@ draw_stencil_pixels(struct gl_context *ctx, GLint x, GLint y,
             case PIPE_FORMAT_S8_UINT:
                {
                   ubyte *dest = stmap + spanY * pt->stride;
-                  assert(usage == PIPE_TRANSFER_WRITE);
+                  assert(usage == PIPE_MAP_WRITE);
                   memcpy(dest, sValues, width);
                }
                break;
@@ -1049,7 +1050,7 @@ draw_stencil_pixels(struct gl_context *ctx, GLint x, GLint y,
                if (format == GL_DEPTH_STENCIL) {
                   uint *dest = (uint *) (stmap + spanY * pt->stride);
                   GLint k;
-                  assert(usage == PIPE_TRANSFER_WRITE);
+                  assert(usage == PIPE_MAP_WRITE);
                   for (k = 0; k < width; k++) {
                      dest[k] = zValues[k] | (sValues[k] << 24);
                   }
@@ -1057,7 +1058,7 @@ draw_stencil_pixels(struct gl_context *ctx, GLint x, GLint y,
                else {
                   uint *dest = (uint *) (stmap + spanY * pt->stride);
                   GLint k;
-                  assert(usage == PIPE_TRANSFER_READ_WRITE);
+                  assert(usage == PIPE_MAP_READ_WRITE);
                   for (k = 0; k < width; k++) {
                      dest[k] = (dest[k] & 0xffffff) | (sValues[k] << 24);
                   }
@@ -1067,7 +1068,7 @@ draw_stencil_pixels(struct gl_context *ctx, GLint x, GLint y,
                if (format == GL_DEPTH_STENCIL) {
                   uint *dest = (uint *) (stmap + spanY * pt->stride);
                   GLint k;
-                  assert(usage == PIPE_TRANSFER_WRITE);
+                  assert(usage == PIPE_MAP_WRITE);
                   for (k = 0; k < width; k++) {
                      dest[k] = (zValues[k] << 8) | (sValues[k] & 0xff);
                   }
@@ -1075,7 +1076,7 @@ draw_stencil_pixels(struct gl_context *ctx, GLint x, GLint y,
                else {
                   uint *dest = (uint *) (stmap + spanY * pt->stride);
                   GLint k;
-                  assert(usage == PIPE_TRANSFER_READ_WRITE);
+                  assert(usage == PIPE_MAP_READ_WRITE);
                   for (k = 0; k < width; k++) {
                      dest[k] = (dest[k] & 0xffffff00) | (sValues[k] & 0xff);
                   }
@@ -1086,7 +1087,7 @@ draw_stencil_pixels(struct gl_context *ctx, GLint x, GLint y,
                   uint *dest = (uint *) (stmap + spanY * pt->stride);
                   GLfloat *destf = (GLfloat*)dest;
                   GLint k;
-                  assert(usage == PIPE_TRANSFER_WRITE);
+                  assert(usage == PIPE_MAP_WRITE);
                   for (k = 0; k < width; k++) {
                      destf[k*2] = zValuesFloat[k];
                      dest[k*2+1] = sValues[k] & 0xff;
@@ -1095,7 +1096,7 @@ draw_stencil_pixels(struct gl_context *ctx, GLint x, GLint y,
                else {
                   uint *dest = (uint *) (stmap + spanY * pt->stride);
                   GLint k;
-                  assert(usage == PIPE_TRANSFER_READ_WRITE);
+                  assert(usage == PIPE_MAP_READ_WRITE);
                   for (k = 0; k < width; k++) {
                      dest[k*2+1] = sValues[k] & 0xff;
                   }
@@ -1147,6 +1148,7 @@ get_color_fp_variant(struct st_context *st)
    key.pixelMaps = ctx->Pixel.MapColorFlag;
    key.clamp_color = st->clamp_frag_color_in_shader &&
                      ctx->Color._ClampFragmentColor;
+   key.lower_alpha_func = COMPARE_FUNC_ALWAYS;
 
    fpv = st_get_fp_variant(st, st->fp, &key);
 
@@ -1176,6 +1178,7 @@ get_color_index_fp_variant(struct st_context *st)
    key.pixelMaps = 0;
    key.clamp_color = st->clamp_frag_color_in_shader &&
                      ctx->Color._ClampFragmentColor;
+   key.lower_alpha_func = COMPARE_FUNC_ALWAYS;
 
    fpv = st_get_fp_variant(st, st->fp, &key);
 
@@ -1440,7 +1443,7 @@ copy_stencil_pixels(struct gl_context *ctx, GLint srcx, GLint srcy,
 {
    struct st_renderbuffer *rbDraw;
    struct pipe_context *pipe = st_context(ctx)->pipe;
-   enum pipe_transfer_usage usage;
+   enum pipe_map_flags usage;
    struct pipe_transfer *ptDraw;
    ubyte *drawMap;
    ubyte *buffer;
@@ -1474,9 +1477,9 @@ copy_stencil_pixels(struct gl_context *ctx, GLint srcx, GLint srcy,
    }
 
    if (_mesa_is_format_packed_depth_stencil(rbDraw->Base.Format))
-      usage = PIPE_TRANSFER_READ_WRITE;
+      usage = PIPE_MAP_READ_WRITE;
    else
-      usage = PIPE_TRANSFER_WRITE;
+      usage = PIPE_MAP_WRITE;
 
    if (st_fb_orientation(ctx->DrawBuffer) == Y_0_TOP) {
       dsty = rbDraw->Base.Height - dsty - height;
@@ -1552,6 +1555,9 @@ blit_copy_pixels(struct gl_context *ctx, GLint srcx, GLint srcy,
    struct gl_pixelstore_attrib pack, unpack;
    GLint readX, readY, readW, readH, drawX, drawY, drawW, drawH;
 
+   if (type == GL_DEPTH_STENCIL_TO_RGBA_NV || type == GL_DEPTH_STENCIL_TO_BGRA_NV)
+      return GL_FALSE;
+
    if (ctx->Pixel.ZoomX == 1.0 &&
        ctx->Pixel.ZoomY == 1.0 &&
        (type != GL_COLOR ||
@@ -1769,11 +1775,22 @@ st_CopyPixels(struct gl_context *ctx, GLint srcx, GLint srcy,
       rbRead = st_renderbuffer(ctx->ReadBuffer->
                                Attachment[BUFFER_STENCIL].Renderbuffer);
       driver_fp = get_drawpix_z_stencil_program(st, GL_FALSE, GL_TRUE);
-   } else {
-      assert(type == GL_DEPTH_STENCIL);
+   } else if (type == GL_DEPTH_STENCIL) {
       rbRead = st_renderbuffer(ctx->ReadBuffer->
                                Attachment[BUFFER_DEPTH].Renderbuffer);
       driver_fp = get_drawpix_z_stencil_program(st, GL_TRUE, GL_TRUE);
+   } else {
+      assert(type == GL_DEPTH_STENCIL_TO_RGBA_NV || type == GL_DEPTH_STENCIL_TO_BGRA_NV);
+      rbRead = st_renderbuffer(ctx->ReadBuffer->
+                               Attachment[BUFFER_DEPTH].Renderbuffer);
+      if (type == GL_DEPTH_STENCIL_TO_RGBA_NV)
+         driver_fp = get_drawpix_zs_to_color_program(st, GL_TRUE);
+      else
+         driver_fp = get_drawpix_zs_to_color_program(st, GL_FALSE);
+      if (!driver_fp) {
+         assert(0 && "operation not supported by CopyPixels implemetation");
+         return;
+      }
    }
 
 
@@ -1871,10 +1888,16 @@ st_CopyPixels(struct gl_context *ctx, GLint srcx, GLint srcy,
    }
 
    /* Create a second sampler view to read stencil */
-   if (type == GL_STENCIL || type == GL_DEPTH_STENCIL) {
+   if (type == GL_STENCIL || type == GL_DEPTH_STENCIL ||
+       type == GL_DEPTH_STENCIL_TO_RGBA_NV || type == GL_DEPTH_STENCIL_TO_BGRA_NV) {
       write_stencil = GL_TRUE;
       if (type == GL_DEPTH_STENCIL)
          write_depth = GL_TRUE;
+      if (type == GL_DEPTH_STENCIL_TO_RGBA_NV || type == GL_DEPTH_STENCIL_TO_BGRA_NV) {
+         write_depth = FALSE;
+         write_stencil = FALSE;
+      }
+
       enum pipe_format stencil_format =
          util_format_stencil_only(pt->format);
       /* we should not be doing pixel map/transfer (see above) */
